--[[
	author: Zax
	content:
		functions: Wli.
		main UI frame: Wli_frameMain
		event frame: Wli_frameMain
		XML config: Wli_cfg_
	todo:
--]]

---------------------------------------------------------------------------------------
------------------------------------- VARIABLES ---------------------------------------
Wli = {}; -- prefixe fonctions
local addonName = ...

local addonVar = {
	["name"] = addonName, -- nécessaire
	["version"] = GetAddOnMetadata(addonName, "Version"), -- ne pas convertir en nombre ici ! -- nécessaire
	["chatColor"] = "D99B1C", -- nécessaire
	["slashCmd"] = "whisplist", -- nécessaire
	["settingsPanel"] = "", -- nom du frame de settings : laisser vide si utilisation du frame de WoW (ou si pas de settings)
	-------- 
	["uiWidth"] = 90, -- px
	["posX"] = 400,
	["posY"] = 600,
	["uiAlpha"] = 0.9,
	["lineFrameName"] = "Wli_playerLine_", -- avec concaténation numLigne
	
	-------- barre titre
	["menuHeight"] = 16, -- px
	["flashColor"] = {0.9, 0.4, 0.3, 0.7}, -- table requise !
	
	-------- liste
	["maxDataLines"] = 15, -- nombre de lignes maxi (mémoire et lignes visibles)
	["maxNumLetters"] = 16, -- nbre caractères affichés dans liste
	["bkColor"] = "555555", -- fond liste
	["mouseOverAlpha"] = 0.2, -- ligne
	["bkAlpha"] = 0.4,
	["vLine"] = 14, -- hauteur en pixel d'un bouton-ligne de la liste
	["fontColor"] = {
		[0] = "E48B05", -- non répondu
		[1] = "white", -- répondu
	},
	["expandOnReceived"] = 1, -- à mettre dans les prefs ?
}

local playersTable = {} -- liste des whisps de cette session

function Wli.getLocal(key)
	----------- pour traduction (voir localization.lua) : ne pas deplacer !!!
	if (WLI_LOCAL_TBL[key] == nil) then return key else return WLI_LOCAL_TBL[key] end
end;

-------------------------------------------- saved et init
WLI_SAVED = {} -- partie commune a tous les persos de tous les serveurs : contient la liste de /w recus
WLI_SAVEDPERCHAR = {}

function Wli.initData(subValue)
	local tbl = {}
	tbl = {
		["version"] = Wli.getVar("version"),
		["debug"] = 0,
		["visible"] = 1,
		["expanded"] = 1,
		["sounds"] = 1,
		["alphasort"] = 0,
		["lines"] = addonVar["maxDataLines"],
		["posX"] = Wli.getVar("posX"),
		["posY"] = Wli.getVar("posY"),
	}
	if (subValue ~= nil) then return tbl[subValue] else return tbl end
end

---------------------------------------------------------------------------------------
--------------------------------------- EVENTS ----------------------------------------
function Wli.eventHandler(self, event,...)
	if (event == "PLAYER_LOGIN") then
		Wli.loading()
	elseif (event == "PLAYER_LOGOUT") then
		Wli.savePlayersList()
	elseif (event == "CHAT_MSG_WHISPER") then
		Wli.updatePlayersReceived(...)
	elseif (event == "CHAT_MSG_WHISPER_INFORM") then
		Wli.updatePlayersSend(...)
--[[
	elseif (event == "CHAT_MSG_BN_WHISPER") then
	elseif (event == "CHAT_MSG_BN_WHISPER_INFORM") then
--]]
	end
end

---------------------------------------------------------------------------------------
--------------------------------- CHARGEMENT & BINDINGS -------------------------------

function Wli.loading()
	----------------------------- 
	Wli_frameMain:RegisterEvent("PLAYER_LOGOUT")
	Wli_frameMain:RegisterEvent("CHAT_MSG_WHISPER")
	Wli_frameMain:RegisterEvent("CHAT_MSG_WHISPER_INFORM")
	Wli_frameMain:RegisterEvent("CHAT_MSG_BN_WHISPER")
	Wli_frameMain:RegisterEvent("CHAT_MSG_BN_WHISPER_INFORM")
	----------------------------- slash commandes
	SlashCmdList["WHISPLISTSHOW"] = Wli.slashCmd
		SLASH_WHISPLISTSHOW1 = "/"..addonVar["slashCmd"]
		SLASH_WHISPLISTSHOW2 = "/wlist"
	
	----------------------------- settings
	if (Wli.isEmpty(WLI_SAVEDPERCHAR)) then
--print("chargement() : === no lua file : INIT SETTINGS ===")--/////
		WLI_SAVEDPERCHAR = Wli.initData()
--Wli.echoTable(CMDLINE_SAVED)--/////
	else
		local tbl = Wli.initData()
		if (Wli.tableKeyDiff(tbl, WLI_SAVEDPERCHAR, "") ~= nil) then
			WLI_SAVEDPERCHAR = Wli.copyTable(tbl, false)
--print("chargement() : === diff saved data INIT SETTINGS ===")--/////
		end
	end

	----------------------------- suite
	Wli.ui_createUI()
	if (Wli.isTrue(WLI_SAVEDPERCHAR["visible"])) then
		Wli_frameMain:Show()
		Wli.ui_expand(WLI_SAVEDPERCHAR["expanded"])
	else
		Wli_frameMain:Hide()
	end
	
	local str = ""
	if (not Wli_frameMain:IsVisible()) then str = " "..Wli.getLocal("Type /whisplist show to display the whisps' list.") end
	if (Wli.debug()) then
		SetCVar("scriptErrors",1)
		str = str..Wli.getTxtColor(" ------ DEBUG ON.", "red")
	end
	if (str ~= "") then Wli.echoChatAppl(str, true, nil) end
end

---------------------------------------------------------------------------------------
-------------------------------------- SLASH CMD --------------------------------------
function Wli.slashCmd(arg, execLevel)
	Wli.slashCmd_Lib(arg, execLevel)
end

function Wli.slashCmd_getList(arg2) -- liste des commandes-slash (voir .lib)
	-- .slashCmd_showList(dontPrint) -- print la liste en print, ou la renvoit en chaine avec dontPrint à true (pour frame cfg)
	local tbl = {
		["user"] = {
			{
				["cmd"] = {"show"},
				["func"] = function() Wli.showHide() end,
				["label"] = Wli.getLocal("show/hide the whisps list"),
			},
			{
				["cmd"] = {"sound", "sounds"},
				["func"] = function() Wli.setSounds(arg2) end,
				["label"] = Wli.getLocal("on | off: turn received whisper sound on or off"),
			},
			{
				["cmd"] = {"reset", "resetui"},
				["func"] = function() Wli.resetui() end,
				["label"] = Wli.getLocal("reset addon interface"),
			},
			{
				["cmd"] = {"resetall"},
				["func"] = function() Wli.resetall() end,
				["label"] = Wli.getLocal("reset all settings to default"),
			},
			{
				["cmd"] = {"help", "aide", "?"},
				["func"] = function() Wli.help() end,
				["label"] = Wli.getLocal("display available slash commands"),
			},
		},
		["dev"] = {
			{
				["cmd"] = {"settings"},
				["func"] = function() Wli.printAddonSettings() end,
				["label"] = "print saved settings",
			},
			{
				["cmd"] = {"auto", "self"},
				["func"] = function() Wli.autoWhisp() end,
				["label"] = "self whisp",
			},
			{
				["cmd"] = {"test"},
				["func"] = function() Wli.fcnTest(arg2) end,
				["label"] = "DEV test with 1 argument",
			},
			{
				["cmd"] = {"fill", "pop"},
				["func"] = function() Wli.fillWithJunk() end,
				["label"] = "fill players list with junk data",
			},
			{
				["cmd"] = {"debug"},
				["func"] = function() Wli.toggleDebug() end,
				["label"] = "toggle debug",
			},
		},
		["DEFAULTCMD"] = "help", -- commande par défaut
		["DEFAULTARG"] = "help", -- argument par défaut d'une commande
	}
	return tbl
end

---------------------------------------------------------------------------------------
--------------------------------------- ACTIONS ---------------------------------------
function Wli.help()
--[[
	local cfgFrame = Ck_cfg_frame -- /////
	local success, errorMessage = pcall(Settings.OpenToCategory, addonName)
	if (not success) then
		InterfaceOptionsFrame_OpenToCategory(cfgFrame) -- wow9
		InterfaceOptionsFrame_OpenToCategory(cfgFrame) -- wow9 (doublé pour ouvrir la page de l'addon)
	end
--]]
	Wli.slashCmd_showList(false) -- pas d'UI (mettre à true pour ne pas printer la liste)
end

function Wli.fcnTest(arg) -- $$$
	print(date("%y.%m.%d %H:%M:%S", GetServerTime()).."===================================")
	print("arg=", arg)
end

function Wli.printAddonSettings()
	Wli.echoTable(WLI_SAVEDPERCHAR, "========== WLI_SAVEDPERCHAR ==========", false,"   ");
	Wli.echo("================================");
end

function Wli.toggleDebug()
	WLI_SAVEDPERCHAR["debug"] = 1 - tonumber(WLI_SAVEDPERCHAR["debug"])
	Wli.echoChatAppl("debug mode "..Wli.booleanToStr(Wli.debug())..".", false, "orange")
end

function Wli.autoWhisp()
	SendChatMessage("=== This is a self-whisper test", "WHISPER", nil, UnitName("player"))
end

function Wli.fillWithJunk()
	wipe(playersTable)
	local ownName, realm = UnitFullName("player")
	local tblJunk = {
		{["time"] = 1, ["name"] = "Ignatus-CenarionCircle", ["replied"] = 0},
		{["time"] = 2, ["name"] = "Arjuna-KirinTor", ["replied"] = 0},
		{["time"] = 10, ["name"] = "Fafner-Garona", ["replied"] = 1},
		{["time"] = 15, ["name"] = "Levo-KirinTor", ["replied"] = 0},
		{["time"] = 91, ["name"] = "Loreleï-KirinTor", ["replied"] = 1},
		{["time"] = 100, ["name"] = "Christian-Malfurion", ["replied"] = 1},
		{["time"] = 502, ["name"] = "Amber-KirinTor", ["replied"] = 0},
		{["time"] = 1100, ["name"] = "Drax-KirinTor", ["replied"] = 0},
		{["time"] = 1200, ["name"] = "Grandulf-KirinTor", ["replied"] = 0},
		{["time"] = 1300, ["name"] = "Drawyn-KirinTor", ["replied"] = 0},
		{["time"] = 1800, ["name"] = ownName.."-"..realm, ["replied"] = 0},
	}
	for i,value in ipairs(tblJunk) do
		table.insert(playersTable, value)
	end
	
	Wli.ui_updateDisplayList(true)
end

function Wli.resetui()
	WLI_SAVEDPERCHAR["posX"] = Wli.getVar("posX")
	WLI_SAVEDPERCHAR["posY"] = Wli.getVar("posY")
	Wli_frameMain:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", WLI_SAVEDPERCHAR["posX"], WLI_SAVEDPERCHAR["posY"])
end

function Wli.resetall()
	wipe(WLI_SAVEDPERCHAR)
	WLI_SAVEDPERCHAR = Wli.initData()
	wipe(playersTable)
	Wli.ui_createUI()
	Wli.showHide(true)
end

function Wli.showHide(optionalBoolean)
	local val
	if (optionalBoolean == nil) then
		----------- switch
		val = (not Wli_frameMain:IsVisible())
	else
		----------- given value
 		val = Wli.isTrue(optionalBoolean, true)
	end
	
	if (val) then
		Wli_frameMain:Show()
		Wli.ui_expand(Wli.isTrue(WLI_SAVEDPERCHAR["expanded"]))
	else
		Wli_frameMain:Hide()
	end
	WLI_SAVEDPERCHAR["visible"] = Wli.booleanToInt(val)
end

function Wli.ui_expand(optionalBoolean)
	if (Wli_frameList ~= nil) then -- nécessaire si appel fcn alors que Wli_frameList n'est pas encore crée
		local val
		if (optionalBoolean == nil) then
			----------- switch
			val = (not Wli_frameList:IsVisible())
		else
			----------- given value
			val = Wli.isTrue(optionalBoolean, true)
		end
		
		if (val) then
			Wli_frameList:Show()
			Wli.ui_showNotClamped()
		else
			Wli_frameList:Hide()
		end
		WLI_SAVEDPERCHAR["expanded"] = Wli.booleanToInt(val)
	end
end

function Wli.isExpanded()
	return Wli_frameList:IsVisible()
end

function Wli.ui_showNotClamped() -- déplace l'ensemble pour montrer la liste si collée au bas de l'écran
	local point, relativeTo, relativePoint, xOfs, yOfs = Wli_frameMain:GetPoint()
--print("1 - point=", point, "---relativeTo=", relativeTo, "---relativePoint=", relativePoint, "---xOfs)=", xOfs, "---yOfs=", yOfs)--/////
	if (string.find(relativePoint, "BOTTOM", 1, true) ~= nil) then
		local neededHeight = Wli_frameMain:GetHeight() + Wli_frameList:GetHeight() + Wli_frameList_resizeBtn:GetHeight() + 5
--print("2 - relativePoint=", relativePoint, "---neededHeight=", neededHeight, "---yOfs=", yOfs, "---(point)=", point, "---(xOfs)=", xOfs)--/////
		if (yOfs < neededHeight) then
			--Wli_frameMain:ClearAllPoints()
			Wli_frameMain:SetPoint(point, "UIParent", relativePoint, xOfs, neededHeight)
			Wli.saveFramePosition()
		end
	end
end

function Wli.loadPrevListe()
	if (not Wli.isEmpty(WLI_SAVED["list"])) then
		if (type(WLI_SAVED["list"][1]) ~= "table") then
			------ compatibilité version
			wipe(WLI_SAVED["list"])
			Wli.echoChatAppl(Wli.getLocal("Incompatible saved list."), false, "red")
		else
			------ OK
			wipe(playersTable)
			playersTable = Wli.copyTable(WLI_SAVED["list"])
			Wli.ui_updateDisplayList(true)
		end
	else
		Wli.echoChatAppl(Wli.getLocal("There is no saved list."), false, "red")
	end
end

function Wli.setSounds(optionalBoolean)
	local val
	if (optionalBoolean == nil) then
		----------- switch
		val = (not Wli.isTrue(WLI_SAVEDPERCHAR["sounds"], true))
	else
		----------- given value
 		val = Wli.isTrue(optionalBoolean, true)
	end
	WLI_SAVEDPERCHAR["sounds"] = Wli.booleanToInt(val)
	Wli.echoChatAppl("sounds "..Wli.booleanToON(WLI_SAVEDPERCHAR["sounds"])..".", false, "orange")
end

function Wli.sortList()-- UNUSED
	WLI_SAVEDPERCHAR["alphasort"] = Wli.inversionValeur(WLI_SAVEDPERCHAR["alphasort"]) -- toggle
	Wli.ui_updateDisplayList(false)
end

function Wli.markAllReplied()
	for i, value in ipairs(playersTable) do
		playersTable[i]["replied"] = 1
	end
	
	Wli.ui_updateDisplayList(false)
end

---------------------------------------------------------------------------------------
-------------------------------------- LISTE ------------------------------------------
function Wli.getNormalizedPlayerName(playerName) -- renvoit le nom du joueur passé AVEC le serveur (pour bug event TBC)
	local thisServer = GetNormalizedRealmName()
	if (playerName ~= nil) then
		if (string.find(playerName, "-", 1, true) == nil) then -- nom serveur pas trouvé
			return playerName.."-"..thisServer
		else
			return playerName
		end
	else
		return "" -- pas nil pour concaténation
	end
end

function Wli.getShortPlayerName(playerName) -- renvoit le nom du joueur passé sans le serveur si sur même serveur
	local thisServer = GetNormalizedRealmName()
	if (playerName ~= nil) then
		if (string.find(playerName, "-"..thisServer, 1, true) ~= nil) then -- même serveur
			return string.gsub(playerName, "-.*", "") -- nom joueur seul si meme serveur
		else
			return playerName
		end
	else
		return "" -- pas nil pour concaténation
	end
end

function Wli.getPlayerLineIndex(playerName) -- renvoit l'index de la ligne correspondant au nom du joueur passé, ou 0
	local numLiFound = 0
	for i, value in ipairs(playersTable) do
		Wli.testPrint("getPlayerLineIndex()--------playerName="..playerName.."---i="..i.."---playersTable[name]="..value["name"].."---")
		if (value["name"] == playerName) then
			numLiFound = i
			break
		end
	end
	return numLiFound
end

function Wli.formatPlayerLine(timeStamp, playerName, repliedBoolean)
	return {["time"] = timeStamp, ["name"] = playerName, ["replied"] = Wli.booleanToInt(Wli.isTrue(repliedBoolean))}
end

function Wli.updatePlayersList(playerName, repliedBoolean) -- MAJ de la liste
	local playerTable = Wli.formatPlayerLine(time(), playerName, repliedBoolean)
	local foundLine = Wli.getPlayerLineIndex(playerName)
	if (Wli.debug()) then print("updatePlayersList(): playerName=", playerName, "---foundLine=", foundLine, "---playerTable[name]=", playerTable["name"], "---") end
	
	if (foundLine > 0) then
		------------------------------------------- joueur deja listé
		playersTable[foundLine] = playerTable
		Wli.testPrint("majListeMurmures()---"..playerName.."---deja dans liste : ligne="..foundLine)
	else
		------------------------------------------- construction table temps et noms
		--------------------- ménage AU DEBUT si table trop grande
		while (#playersTable > addonVar["maxDataLines"]) do
			table.remove(playersTable, 1)
		end
		---------------------
		table.insert(playersTable, playerTable) -- à la fin par défaut
		Wli.testPrint("majListeMurmures()---"..playerName.."---ajouté à la liste")
	end
end

function Wli.updatePlayersReceived(message, sender,...) -- message reçu
--Wli.echo("message="..message.."---sender="..sender.."---language="..Wli.nilSiNul(language).."---channelString="..Wli.nilSiNul(channelString).."---target="..Wli.nilSiNul(target).."---flags="..Wli.nilSiNul(flags).."---unknown="..Wli.nilSiNul(a).."---channelNumber="..Wli.nilSiNul(b).."---channelName="..Wli.nilSiNul(c).."---unknown="..Wli.nilSiNul(d).."---counter="..Wli.nilSiNul(e).."---guid="..Wli.nilSiNul(f).."---"); -- /////
	sender = Wli.getNormalizedPlayerName(sender)
	Wli.testPrint("updatePlayersReceived(): sender="..sender.."---")
	Wli.updatePlayersList(sender, false) -- not replied
	Wli.ui_updateDisplayList(false)
	------------------ effet sonore
	if (Wli.isTrue(WLI_SAVEDPERCHAR["sounds"])) then PlaySound(1204) end -- "PutdownGems"
	------------------ effet visuel
	if ((not Wli.isExpanded()) and (Wli.isTrue(addonVar["expandOnReceived"]))) then Wli.ui_expand(true) end
	Wli.ui_flashUI(0.5, 1, 5) -- 5 flash de 0.5 sec
end

function Wli.updatePlayersSend(message, sender,...) -- message envoyé
	sender = Wli.getNormalizedPlayerName(sender)
	Wli.testPrint("updatePlayersSend(): sender="..sender.."---")
	--Wli.ui_expand(true)
	Wli.updatePlayersList(sender, true) -- replied
	
	Wli.ui_updateDisplayList(false)
end

function Wli.sendWhisp(lineIndex) -- commence un murmure au joueur de la ligne lineIndex
	local dataLine = Wli.getLine(lineIndex)
	if (type(dataLine) == "table") then
		Wli.enterMsg("/w "..dataLine.name.." ")
	else
		Wli.echoChatAppl(dataLine, false, "red") -- chaine d'erreur
	end
end

function Wli.whoJoueur(lineIndex) -- UNUSED
	-- lance une requete who sur le joueur de la ligne lineIndex
end

function Wli.invertPlayerStatus(lineIndex) -- inversion du statut "replied"
	local dataLine = Wli.getLine(lineIndex)
	if (type(dataLine) == "table") then
		local replied = tonumber(dataLine.replied) or 0
		playersTable[lineIndex] = Wli.formatPlayerLine(dataLine.time, dataLine.name, math.abs(replied - 1)) -- inversion replied
		Wli.ui_updateDisplayList(false)
	else
		Wli.echoChatAppl(dataLine, false, "red") -- chaine d'erreur
	end
end

function Wli.deletePlayer(lineIndex)
	local dataLine = Wli.getLine(lineIndex)
	if (type(dataLine) == "table") then
		table.remove(playersTable, lineIndex)
		Wli.ui_updateDisplayList(false)
	else
		Wli.echoChatAppl(dataLine, false, "red") -- chaine d'erreur
	end
end

function Wli.getLine(lineIndex) -- renvoit la ligne lineIndex de la liste des joueurs, ou une chaine si problème
	if (playersTable[lineIndex] ~= nil) then
		local dataLine = playersTable[lineIndex]
		if (type(dataLine) == "table") then
			return dataLine
		else
			return getLocal("Internal error").." [1000]"
		end
	else
		return getLocal("Internal error").." [1001]"
	end
end

---------------------------------------------------------------------------------------
------------------------------------- LIB DEDIEE --------------------------------------

function Wli.debug() -- raccourci : renvoit true si debug ON
	return Wli.isTrue(WLI_SAVEDPERCHAR["debug"])
end

function Wli.getVar(nomVar, subLvl1, subLvl2) -- nécessite addonVar
	-- get valeurs de la table locale addonVar
	-- si aucun argument n'est passé, renvoit la table addonVar
	if (addonVar == nil) then return nil end
	
	if (subLvl2 ~= nil) then
		if (addonVar[nomVar][subLvl1][subLvl2] ~= nil) then return addonVar[nomVar][subLvl1][subLvl2] end
	elseif (subLvl1 ~= nil) then
		if (addonVar[nomVar][subLvl1] ~= nil) then return addonVar[nomVar][subLvl1] end
	elseif (nomVar ~= nil) then
		if (addonVar[nomVar] ~= nil) then return addonVar[nomVar] end
	else
		return addonVar
	end
end

function Wli.setVar(nomVar, valeur) -- nécessite addonVar
	if (addonVar == nil) then return nil end
	if (nomVar ~= nil) then
		if (addonVar[nomVar] ~= nil) then addonVar[nomVar] = valeur end
	end
end

function Wli.getSavedSettings(perCharacterBoolean) -- pour appel depuis autres fichiers
	if (perCharacterBoolean) then return WLI_SAVEDPERCHAR else return WLI_SAVED end
end

---------------------------------------------------------------------------------------
------------------------------------ FCNS INTERNES ------------------------------------

function Wli.savePlayersList()
	-- sauvegarde de la liste actuelle pour la prochaine session
	if (not Wli.isEmpty(playersTable)) then WLI_SAVED["list"] = playersTable end
end

function Wli.saveFramePosition()
	WLI_SAVEDPERCHAR["posX"] = floor(Wli_frameMain:GetLeft() + 0.5);
	WLI_SAVEDPERCHAR["posY"] = floor(Wli_frameMain:GetTop() + 0.5);
	Wli.testPrint("Wli.saveFramePosition: posX="..WLI_SAVEDPERCHAR["posX"].." posY="..WLI_SAVEDPERCHAR["posY"])
end

function Wli.enterMsg(prefixe) -- tape le debut du message dans la Chat Box pour ouvrir un canal
	ChatFrame_OpenChat(prefixe)
	ChatEdit_ParseText(ChatFrame1.editBox, 0)
end

---------------------------------------------------------------------------------------
------------------------------------------- UI ----------------------------------------
function Wli.ui_createUI()
	-------------------------------------------- MAJ du main frame (Wli_frameMain) défini dans .xml (barre de titre)
	Wli_frameMain:SetClampedToScreen(true)
	Wli_frameMain:SetMovable(true)
 	Wli_frameMain:SetUserPlaced(true)
	Wli_frameMain:ClearAllPoints()
	Wli_frameMain:SetFrameStrata("MEDIUM")
	Wli_frameMain:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", Wli.nombreParDefaut(tonumber(WLI_SAVEDPERCHAR["posX"]), addonVar["posX"], 0), Wli.nombreParDefaut(tonumber(WLI_SAVEDPERCHAR["posY"]), addonVar["posY"], 0))
	Wli_frameMain:SetSize(Wli.getVar("uiWidth"), Wli.getVar("menuHeight"))
	--Wli.setColoredFrame(Wli_frameMain, "orange", nil, 0, 0.5)--/////
	
	-------------------------------------------- barre titre (Wli_menuBar) qui recouvre le frameMain
	local menubar = CreateFrame("Button", "Wli_menuBar", Wli_frameMain)
	menubar:SetAllPoints("$parent") -- pas besoin de SetSize()
	----------- textures
	Wli.setTexturedFrame(menubar, "$parent_tex", Wli.getStuffPath("Wli_menubar", true), nil, nil, Wli.getVar("uiAlpha"), 0.2)
	------------- titre
	menubar.text = menubar:CreateFontString("$parent_Text", "OVERLAY", "GameFontNormalSmall")
	Wli.setFontStyle(menubar.text, Wli.getStuffPath("Myriad Condensed Web.ttf", true), 11, {0.3, 0.7, 0.9}, false, 1)
	menubar.text:SetPoint("LEFT", 4, 1)
	menubar.text:SetJustifyH("LEFT")
	menubar.text:SetText(addonVar["name"])
	------------- double-click : expand/collapse
	menubar:SetScript("OnDoubleClick", function(self, mouse)
		if (mouse == "LeftButton") then Wli.ui_expand() end -- switch
	end)
	------------- drag déplacement (clic-gauche avec modificateur, ou clic-droit)
	--menubar:EnableMouse(true)
	menubar:SetScript("OnMouseDown", function(self, mouse)
		if ((mouse == "LeftButton" and IsModifierKeyDown() or mouse == "RightButton") and not menubar.isMoving) then
			self:GetParent():StartMoving()
			self.isMoving = true
		end
	end)
	menubar:SetScript("OnMouseUp", function(self, mouse)
		if (menubar.isMoving) then
			self:GetParent():StopMovingOrSizing()
			self.isMoving = false
			Wli.saveFramePosition()
		end
	end)
	-------------- tooltip
    menubar:SetScript("OnEnter", function(self)
		GameTooltip:SetOwner(self, "ANCHOR_TOP")
		GameTooltip:AddLine(Wli.getLocal("Shift-drag to move the frame.\nDouble-click to collapse/expand."))
		GameTooltip:Show()
    end)
    menubar:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
	end)
	-------------------------------------------- popmenu (MENU TITRE)
	-------------- bouton
	local popupTitleBtn, btnName = Wli.ui_createDownArrow(menubar, 0.5) -- bouton fleche menu : appel par ToggleDropDownMenu()
	popupTitleBtn:SetPoint("RIGHT", menubar, "RIGHT", -5, 0)
	-------------- contenu menu
	local popupMenu_menuBar = CreateFrame("Frame", "Wli_popupMenu_menuBar", nil, "UIDropDownMenuTemplate")
	popupMenu_menuBar.displayMode = "MENU"
	popupMenu_menuBar.initialize = function(self, level)
		local menuContent = {}
		
		menuContent.notCheckable = true
		menuContent.padding = 10 -- /////
		menuContent.isTitle = nil
		menuContent.disabled = nil
	
		menuContent.text = Wli.getLocal("Mark all as replied")
		menuContent.func = function() Wli.markAllReplied() end
		UIDropDownMenu_AddButton(menuContent)
		
		menuContent.text = Wli.getLocal("Load previous list")
		menuContent.func = function() Wli.loadPrevListe() end
		UIDropDownMenu_AddButton(menuContent)
		
		if (WLI_SAVEDPERCHAR["sounds"] == 1) then menuContent.text = Wli.getLocal("Turn off sound") else menuContent.text = Wli.getLocal("Turn on sound") end
		menuContent.func = function() Wli.setSounds() end -- toggle
		UIDropDownMenu_AddButton(menuContent)
		
		if (Wli.isExpanded()) then menuContent.text = Wli.getLocal("Collapse") else menuContent.text = Wli.getLocal("Expand") end
		menuContent.func = function() Wli.ui_expand() end
		UIDropDownMenu_AddButton(menuContent)
		
		menuContent.text = Wli.getLocal("Hide")
		menuContent.func = function()
			Wli.showHide()
			if (not Wli_frameMain:IsVisible()) then Wli.echoChatAppl(Wli.getLocal("hidden. Type /whisplist to show the list again."), false, "yellow") end
		end
		UIDropDownMenu_AddButton(menuContent)
		
		menuContent.text = Wli.getLocal("Cancel")
		menuContent.colorCode = "|cffAAAADD"
		menuContent.func = function()  end
		UIDropDownMenu_AddButton(menuContent)
		
		menuContent.text = Wli.getLocal("Reset")
		menuContent.colorCode = "|cffDC571F"
		menuContent.func = function() Wli.resetall() end
		UIDropDownMenu_AddButton(menuContent)
	end
	-------------- appel menu
	popupTitleBtn:SetScript("OnMouseUp", function(self, mouse)
		if (mouse == "LeftButton") then ToggleDropDownMenu(1, nil, popupMenu_menuBar, btnName, -20, 1) end
	end)
	-------------------------------------------- frame des ligne
	Wli.ui_createBkLines(Wli_frameMain)
end

function Wli.ui_createDownArrow(parent, hilitedAlpha) -- création du bouton d'appel d'un menu popup
	local hilitedAlpha = tonumber(hilitedAlpha) or 0.5
	
	local btnName = parent:GetName().."_popupBtn" -- besoin du nom en retour pour l'ancre du menu popup
	local popupBtn = CreateFrame("Button", btnName, parent)
	popupBtn:SetPoint("RIGHT", parent, "RIGHT", -14, 0)
	popupBtn:SetSize(16, 16)
	popupBtn:SetFrameStrata("MEDIUM")
	popupBtn:SetNormalTexture(Wli.getStuffPath("arrow_down", true))
	popupBtn:SetHighlightTexture(Wli.getStuffPath("arrow_down", true))
	popupBtn:SetPushedTexture(Wli.getStuffPath("arrow_down", true))
	
	return popupBtn, btnName
end

function Wli.ui_createBkLines(parent)
	local listFrame = CreateFrame("Frame", "Wli_frameList", parent)
	Wli.setColoredFrame(listFrame, addonVar["bkColor"], nil, 0, addonVar["bkAlpha"])
	listFrame:SetPoint("TOP", parent, "BOTTOM", 0, 0)
	listFrame:SetSize(parent:GetWidth(), tonumber(WLI_SAVEDPERCHAR["lines"]) * addonVar["vLine"])
	-------------------------------------------- resize
	local function setFrameLimits(frame, minWidth, maxWidth, minHeight, maxHeight) -- pour compat wow9/wow10
		frame:SetMinResize(tonumber(minWidth) or 10, tonumber(minHeight) or 10)--non wow10
		frame:SetMaxResize(tonumber(maxWidth) or 50, tonumber(maxHeight) or 50)--non wow10
	end
    listFrame:SetResizable(true)
	local success, errorMessage = pcall(setFrameLimits, listFrame, parent:GetWidth(), parent:GetWidth(), (3 * addonVar["vLine"]) + 2, addonVar["maxDataLines"] * addonVar["vLine"])
	if (not success) then -- wow10
		listFrame:SetResizeBounds(parent:GetWidth(), (3 * addonVar["vLine"]) + 2, parent:GetWidth(), addonVar["maxDataLines"] * addonVar["vLine"])
	end
	-------------------------------------------- poignée bas : resize
	local resizeBtn = CreateFrame("Button", "$parent_resizeBtn", listFrame)
	resizeBtn:SetSize(64, 8)
	resizeBtn:SetPoint("TOP", listFrame, "BOTTOM", 0, 0)
	resizeBtn:SetNormalTexture(Wli.getStuffPath("hamburger", true))
	resizeBtn:SetHighlightTexture(Wli.getStuffPath("hamburger", true))
	resizeBtn:SetPushedTexture(Wli.getStuffPath("hamburger", true))
	------------- scripts
	resizeBtn:SetScript("OnMouseDown", function(self, mouse)
		if (mouse == "LeftButton" and not listFrame.isResizing) then
			listFrame.isResizing = true
			listFrame:StartSizing("BOTTOM")
		end
	end)
	resizeBtn:SetScript("OnMouseUp", function(self, mouse)
		if (mouse == "LeftButton" and listFrame.isResizing) then
			listFrame:StopMovingOrSizing()
			listFrame.isResizing = false
			listFrame:ClearAllPoints()
			listFrame:SetPoint("TOP", parent, "BOTTOM", 0, 0) -- re-anchor nécessaire après un StartSizing') !!!
			Wli.ui_resizeBkLines(listFrame)
		end
	end)
	
	Wli.ui_createLines(listFrame)
end

function Wli.ui_createLines(parent) -- creation des 15 boutons-lignes
	-------------------------------------------- popmenu sur lignes (MENU LIGNES) : appel par ToggleDropDownMenu()
	-------------- contenu menu
	local popupMenu_lines = CreateFrame("Frame", "Wli_popupMenu_lines", nil, "UIDropDownMenuTemplate")
	popupMenu_lines.displayMode = "MENU"
	popupMenu_lines.initialize = function(self, level)
		local menuContent = {}
	
		menuContent.notCheckable = true
		menuContent.padding = 10 -- /////
		
		menuContent.isTitle = true
		menuContent.text = Wli.getLine(_G[Wli_frameList.frameLineClicked].lineDataIndex).name or "UNKNOWN" -- player name
		--menuContent.text = _G[Wli_frameList.frameLineClicked].text:GetText() -- player name affiché dans menu : tronqué !
		UIDropDownMenu_AddButton(menuContent)
		
		menuContent.isTitle = nil
		menuContent.disabled = nil

		menuContent.text = Wli.getLocal("Whisper")
		menuContent.func = function() Wli.ui_clickInLine(nil,"send") end
		UIDropDownMenu_AddButton(menuContent)
		
	--[[ API SendWho() ne fonctionne plus !
			menuContent.text = Wli.getLocal("Who")
			menuContent.func = function() Wli.ui_clickInLine(nil,"who") end
			UIDropDownMenu_AddButton(menuContent)
	--]]
		
		menuContent.text = Wli.getLocal("Mark as received / replied")
		menuContent.func = function() Wli.ui_clickInLine(nil,"invert") end
		UIDropDownMenu_AddButton(menuContent)
		
		menuContent.text = Wli.getLocal("Delete")
		menuContent.func = function() Wli.ui_clickInLine(nil,"delete") end
		UIDropDownMenu_AddButton(menuContent)
		
		menuContent.text = Wli.getLocal("Cancel")
		menuContent.colorCode = "|cffAAAADD"
		menuContent.func = function()  end
		UIDropDownMenu_AddButton(menuContent)
	end
	
	------------------------------------------- création des frames/lignes
	local fLine
	for i = 1, 15 do
		fLine = CreateFrame("Button", addonVar["lineFrameName"]..i, parent)
		fLine:SetFrameStrata("MEDIUM")
		fLine:SetSize(parent:GetWidth(), addonVar["vLine"])
		fLine:SetPoint("TOPLEFT", parent, "TOPLEFT", 0, 0 - (addonVar["vLine"] * (i - 1)))
		---------------------------------- texture
		Wli.setColoredButton(fLine, addonVar["bkColor"], nil, 0, 0, addonVar["mouseOverAlpha"], nil) -- hilite par alpha
		---------------------------------- texte
		fLine.text = fLine:CreateFontString("$parent_Text", "OVERLAY", "GameFontNormalSmall")
		Wli.setFontStyle(fLine.text, Wli.getStuffPath("Myriad Condensed Web.ttf", true), 11, "grey", false, 1) -- couleur font modifiée sur MAJ liste
		fLine.text:SetNonSpaceWrap(true)
		fLine.text:SetPoint("LEFT", 3, 0)
		fLine.text:SetJustifyH("LEFT")
		---------------------------------- scripts
		fLine:RegisterForClicks("LeftButtonUp", "RightButtonUp")
		fLine:SetScript("OnClick", function(self, mouse)
			Wli_frameList.frameLineClicked = self:GetName() -- pour clic dans ligne (direct ou menu local)
			if (mouse == "RightButton") then
				ToggleDropDownMenu(1, nil, popupMenu_lines, self:GetName(), self:GetWidth() - 40, 47) -- MENU LIGNES
			else
				if (IsModifierKeyDown()) then
					Wli.ui_clickInLine(self, "invert")
				else
					Wli.ui_clickInLine(self, "send")
				end
			end
		end)
		----------------------------------
		fLine:Hide()
	end
end

function Wli.ui_clickInLine(frameLine, action) -- clic liste joueurs
	if (frameLine == nil) then
		frameLine = _G[Wli_frameList.frameLineClicked] -- le clic vient du menu local (de chaque ligne)
	end
	local lineDataIndex = frameLine.lineDataIndex
	if (Wli.debug()) then print("ui_clickInLine() : frameLine=", frameLine:GetName(), "---lineDataIndex=", lineDataIndex, "---action=", action) end

	if (lineDataIndex ~= nil) then -- secu
		if (lineDataIndex > 0 and lineDataIndex <= addonVar["maxDataLines"]) then
			if (action == "invert") then
				Wli.invertPlayerStatus(lineDataIndex)
			elseif (action == "delete") then
				Wli.deletePlayer(lineDataIndex)
			else
				if (action == "send") then
					Wli.sendWhisp(lineDataIndex)
				else
					Wli.whoJoueur(lineDataIndex)
				end
			end
		else
			Wli.echoChatAppl("unexpected error  [3472]", false, "red")
		end
	else
		Wli.echoChatAppl("unexpected error   [3473]", false, "red")
	end
end

function Wli.ui_resizeBkLines(listFrame) -- fin du resize par la poignée bas
	WLI_SAVEDPERCHAR["expanded"] = 1 -- secu
	------------- gestion SetMinResize() et SetMaxResize()
	local minHeight = (3 * addonVar["vLine"]) + 2 -- 3 lignes mini
	local maxHeight = addonVar["maxDataLines"] * addonVar["vLine"]
	local thisHeight = listFrame:GetHeight()
	listFrame:SetHeight(min(maxHeight ,max(minHeight, thisHeight)))
	------------- 
	Wli.ui_defNumBtns(thisHeight) -- maj hauteur liste et nbre boutons
end

function Wli.ui_updateDisplayList(showListBoolean) -- affichage : remplissage des lignes suivant data
	------------------------------------------------ ménage
	for i = addonVar["maxDataLines"], 1, -1 do
		_G[addonVar["lineFrameName"]..i]:Hide()
	end
	----------------------------------------------- affichage lignes
	local playersTblWidth = #playersTable
	local lineFrameIndex = 1
	local numVisibleLines = tonumber(WLI_SAVEDPERCHAR["lines"]) or 5
	local playerShortName = ""
	local frameLine
	local colors = {
		[0] = Wli.getColor(addonVar["fontColor"][0], "table"),
		[1] = Wli.getColor(addonVar["fontColor"][1], "table"),
	}
	
	for lineIndex, playerLine in ipairs(playersTable) do
		if (lineIndex > (playersTblWidth - numVisibleLines)) then
			----------------------- n'afficher que les lignes visibles (suivant hauteur frame liste)
			---------- texte
			playerShortName = Wli.getShortPlayerName(playersTable[lineIndex].name)
			frameLine = _G[addonVar["lineFrameName"]..lineFrameIndex]
			frameLine.text:SetText(Wli.truncStr(playerShortName, addonVar["maxNumLetters"]))
			frameLine.lineDataIndex = lineIndex -- pour .ui_clickInLine()
			---------- colorisation
			Wli.ui_colorizeLine(frameLine, colors[playersTable[lineIndex].replied]) -- 0 ou 1
			----------
			frameLine:Show()
			lineFrameIndex = lineFrameIndex + 1
		end
	end
	----------------------------------------------- dev
	if (Wli.debug()) then
		if (playersTable ~= nil) then
			Wli.echo("-------------------------------- ui_updateDisplayList() : playersTable :", "grey")
			for i, value in ipairs(playersTable) do
				print(i, "---", value["name"], "---", value["replied"])
			end
			Wli.echo("-------------------------------------------------------","grey")
		end
	end
	------------------------------------------------
	if (Wli.isTrue(showListBoolean)) then
		Wli.showHide(true) -- secu
		Wli.ui_expand(true)
	end
end

function Wli.ui_colorizeLine(frameLine, colorTable)
	frameLine.text:SetTextColor(colorTable[1], colorTable[2], colorTable[3])
	--frameLine.texture:SetColorTexture(r, g, b)
end

function Wli.ui_flashUI(delayOn, delayOff, xTime)
	local delayOn = tonumber(delayOn) or 0.5
	local delayOff = tonumber(delayOff) or 1
	local xTime = tonumber(xTime) or 5 -- fois
	
	------------ flash menu bar
	local cpt = 1
	local function flashLoop()
		if (cpt <= xTime) then
--print("ui_flashUI() : cpt=", cpt, "---xTime=", xTime)
			C_Timer.After(delayOff, function()
				Wli.ui_flashMenuBar(delayOn)
				flashLoop()
			end)
			cpt = cpt + 1
		end
	end
	flashLoop()
end

function Wli.ui_flashMenuBar(delayOn)
	local normalTex = Wli_menuBar.texture:GetTexture()
	Wli_menuBar.texture:SetColorTexture(addonVar["flashColor"][1], addonVar["flashColor"][2], addonVar["flashColor"][3], addonVar["flashColor"][4])
	C_Timer.After(delayOn, function()
		Wli_menuBar.texture:SetTexture(normalTex)
	end)
end

function Wli.ui_defNumBtns(hauteurBk)
	local hauteurBk = hauteurBk or 0
	if (hauteurBk < 0) then
		if (not Wli_frameMain:IsVisible()) then Wli_frameMain:Show() end -- secu
		hauteurBk = Wli_frameMain:GetHeight()
	end
	
	WLI_SAVEDPERCHAR["lines"] = floor(hauteurBk / addonVar["vLine"])
	Wli.testPrint("ui_defNumBtns() : hauteurBk="..tostring(hauteurBk).."---nbreLignes="..tostring(WLI_SAVEDPERCHAR["lines"]))
	Wli.ui_updateDisplayList(false)
end
